@@ -1,6 +1,7 @@
 """A corpus package, containing all the basic class and functions to load
-    texts and audio.
+    text, audio and sentences.
 """
 
 from nalp.corpus.audio import AudioCorpus
+from nalp.corpus.sentence import SentenceCorpus
 from nalp.corpus.text import TextCorpus
