@@ -94,24 +94,24 @@ class SeqGAN(Adversarial):
 
         """
 
-        # Creates an optimizer object for pre-training the generator
+        # Creates optimizers for pre-training, discriminator and generator
         self.P_optimizer = pre_optimizer
-
-        # Creates an optimizer object for the discriminator
         self.D_optimizer = d_optimizer
-
-        # Creates an optimizer object for the generator
         self.G_optimizer = g_optimizer
 
         # Defining the loss function
         self.loss = tf.nn.sparse_softmax_cross_entropy_with_logits
 
-        # Defining a loss metric for the discriminator
+        # Defining both loss metrics
         self.D_loss = tf.metrics.Mean(name='D_loss')
-
-        # Defining a loss metric for the generator
         self.G_loss = tf.metrics.Mean(name='G_loss')
 
+        # Storing losses as history keys
+        self.history['pre_D_loss'] = []
+        self.history['pre_G_loss'] = []
+        self.history['D_loss'] = []
+        self.history['G_loss'] = []
+
     def generate_batch(self, batch_size=1, length=1):
         """Generates a batch of tokens by feeding to the network the
         current token (t) and predicting the next token (t+1).
@@ -165,7 +165,7 @@ class SeqGAN(Adversarial):
         """Calculates rewards over an input using a Monte Carlo search strategy.
 
         Args:
-            x (tf.Tensor): A tensor containing the inputs.
+            x (tf.tensor): A tensor containing the inputs.
             n_rollouts (int): Number of rollouts for conducting the Monte Carlo search.
 
         """
@@ -218,8 +218,8 @@ class SeqGAN(Adversarial):
         """Performs a single batch optimization pre-fitting step over the generator.
 
         Args:
-            x (tf.Tensor): A tensor containing the inputs.
-            y (tf.Tensor): A tensor containing the inputs' labels.
+            x (tf.tensor): A tensor containing the inputs.
+            y (tf.tensor): A tensor containing the inputs' labels.
 
         """
 
@@ -246,9 +246,9 @@ class SeqGAN(Adversarial):
         """Performs a single batch optimization step over the generator.
 
         Args:
-            x (tf.Tensor): A tensor containing the inputs.
-            y (tf.Tensor): A tensor containing the inputs' labels.
-            rewards (tf.Tensor): A tensor containing the rewards for the input.
+            x (tf.tensor): A tensor containing the inputs.
+            y (tf.tensor): A tensor containing the inputs' labels.
+            rewards (tf.tensor): A tensor containing the rewards for the input.
 
         """
 
@@ -275,8 +275,8 @@ class SeqGAN(Adversarial):
         """Performs a single batch optimization step over the discriminator.
 
         Args:
-            x (tf.Tensor): A tensor containing the inputs.
-            y (tf.Tensor): A tensor containing the inputs' labels.
+            x (tf.tensor): A tensor containing the inputs.
+            y (tf.tensor): A tensor containing the inputs' labels.
 
         """
 
@@ -331,7 +331,10 @@ class SeqGAN(Adversarial):
                 # Adding corresponding values to the progress bar
                 b.add(1, values=[('loss(G)', self.G_loss.result())])
 
-            logger.file('Loss(G): %s', self.G_loss.result().numpy())
+            # Dump loss to history
+            self.history['pre_G_loss'].append(self.D_loss.result().numpy())
+
+            logger.to_file('Loss(G): %s', self.G_loss.result().numpy())
 
         logger.info('Pre-fitting discriminator ...')
 
@@ -373,14 +376,18 @@ class SeqGAN(Adversarial):
                 # Adding corresponding values to the progress bar
                 b.add(1, values=[('loss(D)', self.D_loss.result())])
 
-            logger.file('Loss(D): %s', self.D_loss.result().numpy())
+            # Dump loss to history
+            self.history['pre_D_loss'].append(self.D_loss.result().numpy())
 
-    def fit(self, batches, epochs=10, d_epochs=5, n_rollouts=16):
+            logger.to_file('Loss(D): %s', self.D_loss.result().numpy())
+
+    def fit(self, batches, epochs=10, g_epochs=1, d_epochs=5, n_rollouts=16):
         """Trains the model.
 
         Args:
             batches (Dataset): Training batches containing samples.
             epochs (int): The maximum number of total training epochs.
+            g_epochs (int): The maximum number of generator epochs per total epoch.
             d_epochs (int): The maximum number of discriminator epochs per total epoch.
             n_rollouts (int): Number of rollouts for conducting the Monte Carlo search.
 
@@ -407,15 +414,17 @@ class SeqGAN(Adversarial):
                 # Gathering the batch size and the maximum sequence length
                 batch_size, max_length = x_batch.shape[0], x_batch.shape[1]
 
-                # Generates a batch of fake inputs
-                x_fake_batch, y_fake_batch = self.generate_batch(
-                    batch_size, max_length)
+                # Iterate through all possible generator's  epochs
+                for _ in range(g_epochs):
+                    # Generates a batch of fake inputs
+                    x_fake_batch, y_fake_batch = self.generate_batch(
+                        batch_size, max_length)
 
-                # Gathers the rewards based on the sampled batch
-                rewards = self._get_reward(x_fake_batch, n_rollouts)
+                    # Gathers the rewards based on the sampled batch
+                    rewards = self._get_reward(x_fake_batch, n_rollouts)
 
-                # Performs the optimization step over the generator
-                self.G_step(x_fake_batch, y_fake_batch, rewards)
+                    # Performs the optimization step over the generator
+                    self.G_step(x_fake_batch, y_fake_batch, rewards)
 
                 # Iterate through all possible discriminator's epochs
                 for _ in range(d_epochs):
@@ -444,5 +453,8 @@ class SeqGAN(Adversarial):
                 b.add(1, values=[('loss(G)', self.G_loss.result()),
                                  ('loss(D)', self.D_loss.result())])
 
-            logger.file('Loss(G): %s| Loss(D): %s', self.G_loss.result(
-            ).numpy(), self.D_loss.result().numpy())
+            # Dumps the losses to history
+            self.history['G_loss'].append(self.G_loss.result().numpy())
+            self.history['D_loss'].append(self.D_loss.result().numpy())
+
+            logger.to_file('Loss(G): %s| Loss(D): %s', self.G_loss.result().numpy(), self.D_loss.result().numpy())
